"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var router_1 = require("@angular/router");
var ayuda_service_1 = require("../service/ayuda.service");
var session_shared_service_1 = require("../service/session-shared.service");
var noop = function () { };
var ɵ0 = noop;
exports.ɵ0 = ɵ0;
exports.CUSTOM_SELECT_CONTROL_VALUE_ACCESSOR = {
    provide: forms_1.NG_VALUE_ACCESSOR,
    useExisting: core_1.forwardRef(function () { return BitSelectComponent; }),
    multi: true
};
/**
* Clase que utiliza un custom componente para select de elementos
* Utiliza la interfaz ControlValueAccessor
*    <bit-select [domain]="'DOMAIN_TIPO_LOG'" id="tipolog" [(ngModel)]="filter.tipoLog" [ngModelOptions]="{standalone: true}">&nbsp;&nbsp;Tipus d'operació</bit-select>
*
* @input data: array de objetos a mostrar en el select. Puede ser null, en cuyo caso domain no puede ser null
* @input domain: nombre del dominio que contiene los datos y que nos resolverá la página cuando cargue el componente. Puede ser null, en cuyo caso data no puede ser null
* @input id: identificador del componente select
* @input filter: true indicará que podemos buscar dentro del select, false que no.
**/
var BitSelectComponent = (function () {
    function BitSelectComponent(router, sessionShared, ayudaService) {
        this.router = router;
        this.sessionShared = sessionShared;
        this.ayudaService = ayudaService;
        this.lectura = false;
        this.readOnly = false;
        this.hideLabel = false;
        this.listaOpciones = [];
        this.emptyText = 'Qualsevol';
        this.includeEmpty = true;
        this.forceObligatorio = false;
        this.onChange = new core_1.EventEmitter();
        this.innerValue = '';
        this.onTouchedCallback = noop;
        this.onChangeCallback = noop;
    }
    BitSelectComponent.prototype.ngOnInit = function () {
        var _this = this;
        //console.log("Vamos a mapear los datos precargados");
        if (this.filter === undefined) {
            this.filter = false;
        }
        if (this.data == null && this.domain != null) {
            //esta manera retornaría los datos alojados en el snapshot pero no tendríamos refresco en el caso de producirse
            //this.tiposLog = this.getTiposLogFromDomain(this.route.snapshot.data[TipoLog.DOMAIN_TIPO_LOG]);//this.route.data.pluck(TipoLog.DOMAIN_TIPO_LOG);
            //esta manera nos permite subscribirnos a los datos de data con lo que si hay refresco nos enteramos
            this.router.data.subscribe(function (result) {
                var dominios = result['dominios'];
                var domainTipo = dominios[_this.domain];
                if (domainTipo != null) {
                    domainTipo.subscribe(function (result) {
                        //console.log(result);
                        //console.log(result);
                        _this.sessionShared.set(_this.domain, result.slice()); // almacenamos en sesión una copia para evitar transmitir cambios
                        _this.listaOpciones = result.slice();
                        _this.postRetrieveData();
                    }, function (error) { _this.listaOpciones = [{ value: null, label: 'Error carrega dominis' }]; });
                }
                else {
                    console.log("No hay datos para el dominio", _this.domain);
                    _this.listaOpciones = [{ value: null, label: 'Error carrega dominis' }];
                }
            });
        }
        else if (this.data) {
            // incluso si la lista de valores ha sido proporcionada,
            // hay que añadir algunos comportamientos posteriores
            this.listaOpciones = this.data.slice();
            this.postRetrieveData();
        }
        //console.log("Componente inicializado");
    };
    BitSelectComponent.prototype.ngOnChanges = function (changes) {
        for (var propName in changes) {
            var changedProp = changes[propName];
            if (propName === 'data') {
                this.listaOpciones = changedProp.currentValue ? changedProp.currentValue.slice() : [];
                this.postRetrieveData();
            }
            else if (propName === 'dominio') {
                this.domain = changedProp.currentValue;
                // TODO: recargar dominio por si es distinto?
            }
        }
    };
    // custom processing after retrieving data array from service
    // custom processing after retrieving data array from service
    BitSelectComponent.prototype.postRetrieveData = 
    // custom processing after retrieving data array from service
    function () {
        var _this = this;
        if (this.includeEmpty) {
            // determinamos si debemos incluir la opción "sin selección" y con qué texto
            this.listaOpciones && this.listaOpciones.splice(0, 0, { value: null, label: this.emptyText });
        }
        // repasamos si hay opciones a excluir que debamos eliminar
        if (this.excludedValues) {
            this.listaOpciones = this.listaOpciones.filter(function (op) { return _this.excludedValues.indexOf(op.value) < 0; });
        }
    };
    Object.defineProperty(BitSelectComponent.prototype, "value", {
        //get accessor
        get: 
        //get accessor
        function () {
            return this.innerValue;
        },
        //set accessor including call the onchange callback
        set: 
        //set accessor including call the onchange callback
        function (v) {
            if (v !== this.innerValue) {
                this.innerValue = v;
            }
            //this.onChangeCallback(v);
        },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(BitSelectComponent.prototype, "label", {
        get: function () {
            var val = this.value;
            var dom = this.listaOpciones && this.listaOpciones.find(function (d) { return d.value == val; });
            return dom ? dom.label : null;
        },
        enumerable: true,
        configurable: true
    });
    BitSelectComponent.prototype.changed = function (event) {
        this.onTouchedCallback();
        this.onChangeCallback(this.innerValue); //TODO: probar que esto es así
        this.onChange.emit(event);
    };
    BitSelectComponent.prototype.onBlur = function () {
        this.onTouchedCallback();
    };
    BitSelectComponent.prototype.onFocus = function () {
        this.onTouchedCallback();
        this.onChangeCallback(this.innerValue);
    };
    BitSelectComponent.prototype.writeValue = function (value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    };
    BitSelectComponent.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    BitSelectComponent.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    Object.defineProperty(BitSelectComponent.prototype, "obligatorio", {
        // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
        // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
        get: 
        // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
        // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
        function () {
            return this.forceObligatorio || (this.control && this.control.errors && this.control.errors['required']);
        },
        enumerable: true,
        configurable: true
    });
    BitSelectComponent.prototype.showAyuda = function () {
        this.ayudaService.mostrarAyuda(this.ayuda);
    };
    BitSelectComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'bit-select',
                    template: "<label *ngIf=\"!hideLabel\"><ng-content></ng-content>&nbsp;<span *ngIf=\"obligatorio\" class=\"obligatorio fa fa-asterisk\"></span>\n            &nbsp;&nbsp;\n                <i *ngIf=\"ayuda\" (click)=\"showAyuda()\" class=\"btn-ayuda fa fa-question\"></i>\n           </label>\n           <p *ngIf=\"lectura || readOnly\" class=\"lectura\">{{label}}</p>\n           <div *ngIf=\"!readOnly\">\n                <p-dropdown *ngIf=\"!lectura\" [options]=\"listaOpciones\" [(ngModel)]=\"value\" [filter]=\"filter\" id=\"{{id}}\" [autoWidth]=\"false\" (onChange)=\"changed(event)\" (onFocus)=\"onFocus()\" (onBlur)=\"onBlur()\">\n                        <ng-template let-item pTemplate=\"item\">\n                            <span [ngClass]=\"{'opcion-desactivada': item.disabled}\">{{item.label}}</span>\n                        </ng-template>\n                </p-dropdown>\n                <control-messages *ngIf=\"control!=null\" [control]=\"control\" [field]=\"id\"></control-messages>\n           </div>\n",
                    providers: [exports.CUSTOM_SELECT_CONTROL_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    BitSelectComponent.ctorParameters = function () { return [
        { type: router_1.ActivatedRoute, },
        { type: session_shared_service_1.SessionSharedService, },
        { type: ayuda_service_1.AyudaService, },
    ]; };
    BitSelectComponent.propDecorators = {
        "control": [{ type: core_1.Input },],
        "id": [{ type: core_1.Input },],
        "filter": [{ type: core_1.Input },],
        "ayuda": [{ type: core_1.Input },],
        "lectura": [{ type: core_1.Input },],
        "readOnly": [{ type: core_1.Input },],
        "excludedValues": [{ type: core_1.Input },],
        "hideLabel": [{ type: core_1.Input },],
        "data": [{ type: core_1.Input },],
        "domain": [{ type: core_1.Input },],
        "emptyText": [{ type: core_1.Input },],
        "includeEmpty": [{ type: core_1.Input },],
        "forceObligatorio": [{ type: core_1.Input },],
        "onChange": [{ type: core_1.Output },],
    };
    return BitSelectComponent;
}());
exports.BitSelectComponent = BitSelectComponent;
//# sourceMappingURL=bit-select.component.js.map